/** @module project/state */

/**
 * state.js
 *
 * Collection of everything which has to do with state changes.
 * - actionTypes
 * - actions
 **/

import combineReducers from 'redux'
import reduxCrudGen from 'redux-gen/crud'
import reduxGen from 'redux-gen/basic'
import { NAME } from './constants'

/**
  *
  */
import http from 'crud-api/http'
import localstorage from 'crud-api/localStorage'
const basic = reduxGen(NAME)
const db = reduxCrudGen(NAME, 'db', http)
const local = reduxCrudGen(NAME, 'local', localstorage)
console.log('basic, crud', basic, db, local)

/** actionTypes define what actions are handeled by the reducer. */
export const actions = { ...basic.actions, ...db.actions, ...local.actions }
console.log('State actions', actions)

/** state definition */
/** It is generally easier to not have another object here. */
export const state = { ...basic.state, ...db.state, ...local.state }
console.log('State state', state)

/** reducer is called by the redux dispatcher and handles all component actions */
export function reducer (state = [], action) {
  let nextState = state
  nextState = basic.reducer(nextState, action)
  nextState = db.reducer(nextState, action)
  nextState = local.reducer(nextState, action)
  return nextState
}

/** sagas are asynchronous workers (JS generators) to handle the state. */
export function * saga () {
  yield db.saga()
  yield local.saga()
}
